/***************************************************************************
 *   Copyright (C) 2007 by Antony Martin                                   *
 *   martin.antony@yahoo.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Cree le : 11 fevrier 2007
   derniere modification le 1/05/2007 */


#ifndef __SCECSHADER_H__
#define __SCECSHADER_H__


/* constantes propres a CShader */
#define SCE_VERTEX_SHADER 1
#define SCE_PIXEL_SHADER 2


/* structure d'un shader GLSL */
typedef struct sce_cshaderglsl SCE_CShaderGLSL;
struct sce_cshaderglsl
{
    /* identifiant opengl */
    GLuint id;
    
    /* type du shader (vertex ou fragment) */
    GLenum type;
    
    /* donnes */
    GLchar **data;
    
    /* nombre de chaines (donnees) */
    size_t n_strs;
    
    /* booleen... qui parle de lui meme */
    int is_pixelshader;
    
    /* booleen, true: la source du shader a ete compilee */
    int compiled;
};


/* structure d'un shader ASM */
typedef struct sce_cshaderasm SCE_CShaderASM;
struct sce_cshaderasm
{
    /* identifiant opengl */
    GLuint id;
    
    /* type du shader (vertex ou fragment) */
    GLenum type;
    
    /* donnes */
    GLchar *data;
    
    /* booleen... qui parle de lui meme */
    int is_pixelshader;
    
    /* booleen, true: la source du shader a ete compilee */
    int compiled;
};


#ifdef _SCE_USE_CG
/* structure d'un shader Cg */
typedef struct sce_cshadercg SCE_CShaderCG;
struct sce_cshadercg
{
    /* identifiant cg */
    CGprogram id;
    
    /* type du shader (vertex / fragment), ou 'profile' pour Cg */
    CGprofile type;
    
    /* donnes */
    char *data;
    
    /* arguments du shader */
    char **args;
    
    /* booleen... qui parle de lui meme */
    int is_pixelshader;
    
    /* booleen, true: la source du shader a ete compilee */
    int compiled;
};
#endif



/* structure d'un programme regroupant des shaders GLSL */
typedef struct sce_cprogram SCE_CProgram;
struct sce_cprogram
{
    /* identifiant opengl */
    GLuint id;
    
    /* booleen indiquant si le programme a ete lie/relie
       depuis la dernire assignation de shader au program */
    int compiled;
    /* eventuellement, rajouter des donnees intelligentes pour le stockage
       intelligent de Uniform* ... */
};



/* initialise le gestionnaire de shaders
   le parametre est un booleen indiquant si l'on souhaite ou non utiliser Cg */
int SCE_CInitShaders(int);

/* quitte le gestionnaire de shaders */
void SCE_CQuitShaders(void);

#ifdef _SCE_USE_CG
/* renvoie le profile par defaut du type (pixel/vertex) demande */
CGprofile SCE_CGetCgProfile(SCEenum);
#endif

/* cree un objet de shader */
SCE_CShaderGLSL* SCE_CCreateShaderGLSL(SCEenum);
SCE_CShaderASM* SCE_CCreateShaderASM(SCEenum);
#ifdef _SCE_USE_CG
SCE_CShaderCG* SCE_CCreateShaderCG(SCEenum);
#endif

/* supprime un shader */
void SCE_CDeleteShaderGLSL(SCE_CShaderGLSL*);
void SCE_CDeleteShaderASM(SCE_CShaderASM*);
#ifdef _SCE_USE_CG
void SCE_CDeleteShaderCG(SCE_CShaderCG*);
#endif

/* associe un code source au shader specifie */
int SCE_CSetShaderGLSLSource(SCE_CShaderGLSL*, char*);
int SCE_CSetShaderASMSource(SCE_CShaderASM*, char*);
#ifdef _SCE_USE_CG
int SCE_CSetShaderCGSource(SCE_CShaderCG*, char*);
/* petite specialisation pour Cg, la gestion des arguments */
int SCE_CSetShaderCGArgs(SCE_CShaderCG*, char**);
#endif

/* construit et compile le shader specifie */
int SCE_CBuildShaderGLSL(SCE_CShaderGLSL*);
int SCE_CBuildShaderASM(SCE_CShaderASM*);
#ifdef _SCE_USE_CG
int SCE_CBuildShaderCG(SCE_CShaderCG*);
#endif

/* definit le shader selectionne comme shader actif pour le rendu */
void SCE_CUseShaderASM(SCE_CShaderASM*);
#ifdef _SCE_USE_CG
void SCE_CUseShaderCG(SCE_CShaderCG*);
#endif


/** !! ** fonctions pour les shaders GLSL uniquement ** !! **/

/* cree un objet de programme */
SCE_CProgram* SCE_CCreateProgram(void);

/* supprime un programme */
void SCE_CDeleteProgram(SCE_CProgram*);

/* associe un shader a un programme */
int SCE_CSetProgramShader(SCE_CProgram*, SCE_CShaderGLSL*, int);

/* lie un programme a ses shaders associes (le construit) */
int SCE_CBuildProgram(SCE_CProgram*);

/* defini le programme (GLSL) actif */
void SCE_CUseProgram(SCE_CProgram*);

/** !! ** ****************************************** ** !! **/


/* fonctions de gestion des etats des shaders
   (activation/desactivation manuelle) */
#if 0
void SCE_CDisableShaderGLSL(void);
void SCE_CDisableShaderASM(int);
#ifdef _SCE_USE_CG
void SCE_CDisableShaderCG(int);
#endif
#endif

/* fonctions d'envoie des parametres.
   attention: pour les shaders GLSL, le mot 'Shader' est remplace par 'Program',
   car les modifications s'effectuent sur un program et non sur un shader. */

/* fonctions renvoyant l'index du parametre dont on donne le nom */
SCEint SCE_CGetProgramIndex(SCE_CProgram*, const char*);
#ifdef _SCE_USE_CG
CGparameter SCE_CGetShaderCGIndex(SCE_CShaderCG*, const char*);
#endif




/* fonctions d'envoie de parametres aux shaders */
/* GLSL */
void SCE_CSetProgramParam(SCEint, int);
void SCE_CSetProgramParamf(SCEint, float);
void SCE_CSetProgramParamfv(SCEint, size_t, const float*);
void SCE_CSetProgramMatrix(SCEint, size_t, const float*);

/* ASM  */
void SCE_CSetShaderASMParamf(SCEuint, float, float, float, float);
void SCE_CSetShaderASMParamfv(SCEuint, size_t, const float*);

/* Cg */
#ifdef _SCE_USE_CG
void SCE_CSetShaderCGParam(CGparameter, int);
void SCE_CSetShaderCGParamf(CGparameter, float);
void SCE_CSetShaderCGParamfv(CGparameter, size_t, const float*);
void SCE_CSetShaderCGMatrix(CGparameter, const float*);
#endif


#endif /* guard */
