/***************************************************************************
 *   Copyright (C) 2007 by Antony Martin                                   *
 *   martin.antony@yahoo.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Cree le : 11 mars 2007
   derniere modification le 18/03/2007 */

#ifndef __SCETEXTURE_H__
#define __SCETEXTURE_H__


/* constantes propres a Texture */
/* filtres de redimensionnement */ /* NOTE: noms constantes a revoir (conflit avec SCE_TEX_NEAREST & cie) */
#define SCE_FILTER_NEAREST      ILU_NEAREST
#define SCE_FILTER_LINEAR       ILU_LINEAR
#define SCE_FILTER_BILINEAR     ILU_BILINEAR
#define SCE_FILTER_BOX          ILU_SCALE_BOX
#define SCE_FILTER_TRIANGLE     ILU_SCALE_TRIANGLE
#define SCE_FILTER_BELL         ILU_SCALE_BELL
#define SCE_FILTER_BSPLINE      ILU_SCALE_BSPLINE
#define SCE_FILTER_LANCZOS3     ILU_SCALE_LANCZOS3
#define SCE_FILTER_MITCHELL     ILU_SCALE_MITCHELL


/* structure d'une image */
typedef struct sce_stexdata SCE_STexData;
struct sce_stexdata
{
    ILuint id;              /* identifiant devil */
    
    unsigned int n_data;    /* nombre de donnees */ /* mipmap (1) */
    SCE_CTexData *cdata;    /* donnees cote coeur */
    
    int freetexels;         /* booleen, true: les texels contenus dans cdata
                                              doivent etre liberes (SCE_free) */
};

/* structure d'une texture, contenant son(ses) image(s) (cube map) */
typedef struct sce_stexture SCE_STexture;
struct sce_stexture
{
    SCE_CTexture *tex;      /* texture du "coeur" */
    
    unsigned int n_data;    /* nombre de donnees */ /* cube map (1) */
    SCE_STexData **data;    /* donnees */
    
    int loaded;             /* booleens parlant d'eux-meme */
    int builded;
};

/* (1)*: plus pratique au niveau des loaders.
         une image d'une face du cube peut etre constituee de plusieurs
         niveaux de mipmap, alors qu'un niveau de mipmap ne peux pas
         representer plusieurs faces de la cubemap a la fois */


/* renvoie d'identifiant des textures pour le media manager */
int SCE_Texture_MediaTypeID(void);

/* initialise le gestionnaire de textures */
int SCE_Texture_Init(void);

/* quitte le gestionnaire de textures */
int SCE_Texture_Quit(void);


/* cree une texture en prenant son type */
SCE_STexture* SCE_Texture_Create(SCEenum);

/* supprime une texture */
void SCE_Texture_Delete(SCE_STexture*);

#if 0
/* recherche le type (enumerateur IL) d'une image d'apres son extension */
SCEenum SCE_Texture_GetImageType(const char*);
#endif

/* defini le filtre a appliquer a l'image lors du redimensionnement */
void SCE_Texture_SetResizeFilter(SCEenum);

/* defini si la texture sera agrandie ou retrecie lors du redimensionnement */
void SCE_Texture_ReduceImages(int);

/* fonctions de chargement d'images */
/* fonction de chargement des fichiers en utilisant DevIL */
void* SCE_Texture_DevILLoader(FILE*, const char*, void*);
/* chargement des images au format DDS */
void* SCE_Texture_DDSLoader(FILE*, const char*, void*);

/* ajoute une donnee image a une texture */
int SCE_Texture_AddImage(SCE_STexture*, SCEenum, SCE_STexData*);


/* construit la texture */
int SCE_Texture_Build(SCE_STexture*, int);

/* met a jour les donnees de la texture */
int SCE_Texture_Update(SCE_STexture*);

/* defini la texture active et son unite de texture */
void SCE_Texture_Use(unsigned int, SCE_STexture*);


#endif /* guard */
