/***************************************************************************
 *   Copyright (C) 2007 by Antony Martin                                   *
 *   martin.antony@yahoo.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Cree le : 12 octobre 2006
   derniere modification le 1/05/2007 */



#include <SCE/SCEngine.h>


int SCE_Init(FILE *outlog, SCEflags flags)
{
    if(outlog)
        Logger_Init(outlog);
    else
    {
        fputs("invalid output stream of errors", stderr);
        return (SCE_ERROR);
    }
    
    /* initialisation du coeur */
    if(SCE_CInit(flags) < 0)
    {
        Logger_LogFinish("can't initialize core!");
        return (SCE_ERROR);
    }
    /***/
    
    /* il est important d'initialiser le gestionnaire de medias avant
       le gestionnaire de textures, car l'initialisation de ce dernier
       fait un appel au media manager */
    
    /* initialisation du gestionnaire de medias */
    if(SCE_Media_Init() < 0)
    {
        SCE_CQuit();
        Logger_LogFinish("can't initialize media manager");
        return (SCE_ERROR);
    }
    /***/
    
    /* initialisation du gestionnaire de textures */
    if(SCE_Texture_Init() < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize texture manager");
        return (SCE_ERROR);
    }
    /***/
    
    /* initialisation du gestionnaire de shaders */
    if(SCE_Shader_Init() < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize texture manager");
        return (SCE_ERROR);
    }
    /***/
    
    return (SCE_OK);
}

int SCE_Quit(void)
{
    if(SCE_Shader_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_Texture_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_Media_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_CQuit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    return (SCE_OK);
}
